"""
pyEIT: Python based Electrical Impedance Tomography

eit:
    reconstruction algorithms
mesh:
    create triangle(2D) / tetrahedron(3D) meshes
"""
__author__ = "Benyuan Liu"
__author_email__ = "liubenyuan@gmail.com"
__license__ = "BSD"
APP_NAME = "pyeit"
APP_VERSION = "1.1.8"
APP_DESCRIPTION = "A Python Package for EIT"
